@@ -1,11 +1,12 @@
-from starlette.applications import Starlette
-
-from dagster import DagsterInstance
-from dagster import _check as check
+from dagster import (
+    DagsterInstance,
+    _check as check,
+)
 from dagster._cli.workspace.cli_target import get_workspace_process_context_from_kwargs
 from dagster._core.execution.compute_logs import warn_if_compute_logs_disabled
 from dagster._core.telemetry import log_workspace_stats
 from dagster._core.workspace.context import WorkspaceProcessContext
+from starlette.applications import Starlette
 
 from .version import __version__
 from .webserver import DagitWebserver
