@@ -3,19 +3,20 @@ import os
 from typing import Optional
 
 import click
-import uvicorn
-
 import dagster._check as check
+import uvicorn
 from dagster._cli.utils import get_instance_for_service
 from dagster._cli.workspace import (
     get_workspace_process_context_from_kwargs,
     workspace_target_argument,
 )
 from dagster._cli.workspace.cli_target import WORKSPACE_TARGET_WARNING
+from dagster._core.instance import InstanceRef
 from dagster._core.telemetry import START_DAGIT_WEBSERVER, log_action
 from dagster._core.telemetry_upload import uploading_logging_thread
 from dagster._core.workspace.context import WorkspaceProcessContext
-from dagster._utils import DEFAULT_WORKSPACE_YAML_FILENAME
+from dagster._serdes import deserialize_as
+from dagster._utils import DEFAULT_WORKSPACE_YAML_FILENAME, find_free_port, is_port_in_use
 from dagster._utils.log import configure_loggers
 
 from .app import create_app_from_workspace_process_context
@@ -30,6 +31,7 @@ DEFAULT_DAGIT_HOST = "127.0.0.1"
 DEFAULT_DAGIT_PORT = 3000
 
 DEFAULT_DB_STATEMENT_TIMEOUT = 15000  # 15 sec
+DEFAULT_POOL_RECYCLE = 3600  # 1 hr
 
 
 @click.command(
@@ -66,8 +68,8 @@ DEFAULT_DB_STATEMENT_TIMEOUT = 15000  # 15 sec
     "--port",
     "-p",
     type=click.INT,
-    help="Port to run server on.",
-    default=DEFAULT_DAGIT_PORT,
+    help=f"Port to run server on - defaults to {DEFAULT_DAGIT_PORT}",
+    default=None,
     show_default=True,
 )
 @click.option(
@@ -80,16 +82,30 @@ DEFAULT_DB_STATEMENT_TIMEOUT = 15000  # 15 sec
 )
 @click.option(
     "--db-statement-timeout",
-    help="The timeout in milliseconds to set on database statements sent "
-    "to the DagsterInstance. Not respected in all configurations.",
+    help=(
+        "The timeout in milliseconds to set on database statements sent "
+        "to the DagsterInstance. Not respected in all configurations."
+    ),
     default=DEFAULT_DB_STATEMENT_TIMEOUT,
     type=click.INT,
     show_default=True,
 )
+@click.option(
+    "--db-pool-recycle",
+    help=(
+        "The maximum age of a connection to use from the sqlalchemy pool without connection"
+        " recycling. Set to -1 to disable. Not respected in all configurations."
+    ),
+    default=DEFAULT_POOL_RECYCLE,
+    type=click.INT,
+    show_default=True,
+)
 @click.option(
     "--read-only",
-    help="Start Dagit in read-only mode, where all mutations such as launching runs and "
-    "turning schedules on/off are turned off.",
+    help=(
+        "Start Dagit in read-only mode, where all mutations such as launching runs and "
+        "turning schedules on/off are turned off."
+    ),
     is_flag=True,
 )
 @click.option(
@@ -106,43 +122,65 @@ DEFAULT_DB_STATEMENT_TIMEOUT = 15000  # 15 sec
         ["critical", "error", "warning", "info", "debug", "trace"], case_sensitive=False
     ),
 )
+@click.option(
+    "--code-server-log-level",
+    help="Set the log level for any code servers spun up by dagit.",
+    show_default=True,
+    default="info",
+    type=click.Choice(
+        ["critical", "error", "warning", "info", "debug", "trace"], case_sensitive=False
+    ),
+)
+@click.option(
+    "--instance-ref",
+    type=click.STRING,
+    required=False,
+    hidden=True,
+)
 @click.version_option(version=__version__, prog_name="dagit")
 def dagit(
     host,
     port,
     path_prefix,
     db_statement_timeout,
+    db_pool_recycle,
     read_only,
     suppress_warnings,
     log_level,
+    code_server_log_level,
+    instance_ref,
     **kwargs,
 ):
     if suppress_warnings:
         os.environ["PYTHONWARNINGS"] = "ignore"
 
-    with get_instance_for_service("dagit") as instance:
+    configure_loggers()
+    logger = logging.getLogger("dagit")
+
+    with get_instance_for_service(
+        "dagit",
+        instance_ref=deserialize_as(instance_ref, InstanceRef) if instance_ref else None,
+        logger_fn=logger.info,
+    ) as instance:
         # Allow the instance components to change behavior in the context of a long running server process
-        instance.optimize_for_dagit(db_statement_timeout)
+        instance.optimize_for_dagit(db_statement_timeout, db_pool_recycle)
 
         with get_workspace_process_context_from_kwargs(
             instance,
             version=__version__,
             read_only=read_only,
             kwargs=kwargs,
+            code_server_log_level=code_server_log_level,
         ) as workspace_process_context:
             host_dagit_ui_with_workspace_process_context(
-                workspace_process_context,
-                host,
-                port,
-                path_prefix,
-                log_level,
+                workspace_process_context, host, port, path_prefix, log_level
             )
 
 
 def host_dagit_ui_with_workspace_process_context(
     workspace_process_context: WorkspaceProcessContext,
     host: Optional[str],
-    port: int,
+    port: Optional[int],
     path_prefix: str,
     log_level: str,
 ):
@@ -150,14 +188,20 @@ def host_dagit_ui_with_workspace_process_context(
         workspace_process_context, "workspace_process_context", WorkspaceProcessContext
     )
     host = check.opt_str_param(host, "host", "127.0.0.1")
-    check.int_param(port, "port")
+    check.opt_int_param(port, "port")
     check.str_param(path_prefix, "path_prefix")
 
-    configure_loggers()
     logger = logging.getLogger("dagit")
 
     app = create_app_from_workspace_process_context(workspace_process_context, path_prefix)
 
+    if not port:
+        if is_port_in_use(host, DEFAULT_DAGIT_PORT):
+            port = find_free_port()
+            logger.warning(f"Port {DEFAULT_DAGIT_PORT} is in use - using port {port} instead")
+        else:
+            port = DEFAULT_DAGIT_PORT
+
     logger.info(
         "Serving dagit on http://{host}:{port}{path_prefix} in process {pid}".format(
             host=host, port=port, path_prefix=path_prefix, pid=os.getpid()
